/*********************************************************************
 * [ODBC経由でExcelブック／CSVにアクセス] 0.1
 *
 * ODBCドライバ経由で、Excelブックファイル／CSVファイルにアクセスし、
 * SQLによる検索を実行します。
 * -------------------------------------------------------------------
 * 2014/06/28 ver 0.1   新規作成
 * -------------------------------------------------------------------
 * Copyright(C) 2014 ASANO Sei (http://sei.qee.jp/docs/)
 *********************************************************************/
var currentOdbcItem = null;
window.resizeTo(699, 660);
/*********************************************************************
 * 初期処理
 * -------------------------------------------------------------------
 * [引数] なし
 * [戻り] なし
 *********************************************************************/
function initialize() {
	editSelectBoxOdbcDriver();
	changeOdbcDriver();
	//TODO configの読み込み
	//TODO ボタンの押下可否を設定。初期状態ではすべて押下不可
}
//ODBCドライバリストの初期化
function editSelectBoxOdbcDriver() {
	var selectBoxOdbcDriver = document.getElementsByName('odbcDriver')[0];
	for (var caption in odbcDriverList) {
		var opt = dom_generateOption(odbcDriverList[caption].DRIVER_NAME, caption);
		opt.targetIsFile = odbcDriverList[caption].TARGET_IS_FILE;
		selectBoxOdbcDriver.appendChild(opt);
	}
}
/*********************************************************************
 * 終了処理
 * -------------------------------------------------------------------
 * [引数] なし
 * [戻り] なし
 *********************************************************************/
function finalize() {
	closeConnection();
	//TODO configの保存
}
/*********************************************************************
 * 各種入力値変更時の処理
 *********************************************************************/
/*--------------------------------------------------------------------
 * ODBCドライバ選択変更時の処理
 *-------------------------------------------------------------------*/
function changeOdbcDriver() {
	var opt = dom_getSelectedOption(null, 'odbcDriver');
	currentOdbcItem = null;
	if (!opt) return;
	currentOdbcItem = opt;
	odbcDriverName.innerHTML = opt.value;
	targetType.innerHTML = 'データ' +  (opt.targetIsFile ? 'ファイル' : 'フォルダ');
	changeOpenButtonStatus();
}
/*--------------------------------------------------------------------
 * ファイル／フォルダパスが変更されたときの処理
 *-------------------------------------------------------------------*/
function changeTargetFullPath() {
	targetFullPath.value = targetFullPath.value.trim();
	changeOpenButtonStatus();
}
//ファイル／フォルダが指定されていれば「開く」を押下可に
function changeOpenButtonStatus() {
	btnOpenDB.disabled = true;
	if (!targetFullPath.value) return;
	if (!currentOdbcItem.targetIsFile) {
		if (fileSystem.fso.folderExists(targetFullPath.value)) btnOpenDB.disabled = false;
		return;
	}
	if (!fileSystem.fso.fileExists(targetFullPath.value)) return;
	//拡張子のチェック
	var ext = fileSystem.fso.getExtensionName(targetFullPath.value); //ピリオドなし
	var re = new RegExp('\\*\.' + ext + '[;,\)]', 'i');
	if (!currentOdbcItem.value.match(re)) return;
	btnOpenDB.disabled = false;
}
/*--------------------------------------------------------------------
 * 開く／閉じるボタンの押下可否を排他的に変更する。
 * 閉じるボタンの押下可否は、そのままSQL実行ボタンの押下可否となる。
 * -------------------------------------------------------------------
 * [引数] src    : 押下可否を設定するボタン。開くボタンまたは閉じるボタン
 *        isDisabled : 押下不可のときtrue
 * [戻り] なし
 *-------------------------------------------------------------------*/
function setOpenCloseButtonDisableExclusive(src, isDisabled) {
	src.disabled = isDisabled;
	var target = (src == btnOpenDB ? btnCloseDB : btnOpenDB);
	target.disabled = !isDisabled;
	btnSchema.disabled = btnExecuteSQL.disabled = btnCloseDB.disabled;
}
/*--------------------------------------------------------------------
 * ファイル／フォルダがドロップされたときの処理
 *-------------------------------------------------------------------*/
function dropAction(data) {
	if (!data || !data.Files.Count) return;
	targetFullPath.value = data.Files(1);
	changeTargetFullPath();
}
/*--------------------------------------------------------------------
 * 「開く」ボタン押下時の処理
 *-------------------------------------------------------------------*/
function openDB() {
	var isOpen = openConnection(generateConnectString());
	if (!isOpen) return;
	var tableNames = getTableNameList();
	var tableNamesArea = dom_removeAllChildren(null, 'tableNamesArea');
	for (var i=0 ; i<tableNames.length ; i++) {
		var tableName = tableNames[i];
		tableNamesArea.appendChild(dom_generateOption(tableName, tableName));
	}

	//開くボタン押下不可
	setOpenCloseButtonDisableExclusive(btnOpenDB, btnCloseDB, true);
	//各種条件を変更不可に設定
	changeStatusConditions(true);
}
//各種条件の変更可否を設定
function changeStatusConditions(isDisabled) {
	dom_getOneElement(null, 'odbcDriver').disabled = isDisabled;
	targetFullPath.disabled = isDisabled;
	ddObj.style.display = isDisabled ? 'none' : ''; //D&Dオブジェクトにdisabledは効かないようなので
	checkBoxHeaderColumn.disabled = isDisabled;
	checkBoxReadOnly.disabled = isDisabled;
}
/*--------------------------------------------------------------------
 * 「閉じる」ボタン押下時の処理
 *-------------------------------------------------------------------*/
function closeDB() {
	closeConnection();
	//閉じるボタン押下不可
	setOpenCloseButtonDisableExclusive(btnCloseDB, btnOpenDB, true);
	//各種条件を変更可に戻す
	changeStatusConditions(false);
	msgArea.innerHTML = '&nbsp;';
}
/*--------------------------------------------------------------------
 * 「SQL実行」ボタン押下時の処理
 *-------------------------------------------------------------------*/
function executeSQL() {
	//範囲選択されていればそれを取得
	sqlTextArea.focus(); //sqlTextAreaが選択されてない場合は要フォーカス
	var textRange = sqlTextArea.document.selection.createRange(); //IEのみ有効の方法
	var str = textRange.text.trim();
	//範囲選択されていなければ全部取得
	if (!str.length) {
		str = sqlTextArea.value.trim();
		if (!str.length) return;
	}
	//SQL取得。複数行にまたがっていたら半角空白で連結。末尾に;がなければ付加。
	var lines = str.splitLine();
	var sql = lines.join(' ');
	if (sql.slice(-1) != ';') sql += ';';
	//select/update/delete等の別をチェック
	var command = null;
	if (sql.match(/^(\S+)\s/i)) command = RegExp.$1.toLowerCase();
	if (!command) return;
	if (checkBoxReadOnly.checked && command != 'select') {
		alert('読込専用で開いているDBに、UPDATE/DELETE処理はできません。');
		return;
	}
	if (command == 'select') {
		editResultTable(executeSelectSql(sql), true);
	} else {
		//TODO update/delete
	}
}
/*--------------------------------------------------------------------
 * 結果テーブルの編集処理
 * -------------------------------------------------------------------
 * [引数] items : 検索結果
 *        seeCheckBox:チェックボックスのチェック状態でヘッダの有無を判断するか
 * [戻り] なし
 *-------------------------------------------------------------------*/
function editResultTable(items, seeCheckBox) {
	dom_removeAllChildren('resultTable', null);
	if (!items || !items.length) return;
	//ヘッダ行の処理
	var includeHeader = false;
	if (seeCheckBox) {
		includeHeader = (checkBoxHeaderColumn.checked);
	} else {
		includeHeader = true;
	}
	if (includeHeader) {
		var thead = dom_generateElement('THEAD');
		resultTable.appendChild(thead);
		var tr = dom_generateElement('TR');
		thead.appendChild(tr);
		tr.appendChild(dom_generateElement('TH', '#'));
		for (var i=0 ; i<items[0].length ; i++) {
			tr.appendChild(dom_generateElement('TH', items[0][i]));
		}
	}
	//データ行の処理
	var tbody = dom_generateElement('TBODY');
	resultTable.appendChild(tbody);
	var startIndex = 0;
	if (includeHeader) startIndex++;
	for (var i=startIndex ; i<items.length ; i++) {
		var tr = dom_generateElement('TR');
		tbody.appendChild(tr);
		tr.appendChild(dom_generateElement('TD', i));
		for (var j=0 ; j<items[i].length ; j++) {
			var item = items[i][j];
			tr.appendChild(dom_generateElement('TD', (item==null ? '' : item)));
		}
	}
	//件数
	msgArea.innerHTML = (includeHeader ? items.length - 1 : items.length) + '件検索';
}
/*--------------------------------------------------------------------
 * 「Schema」ボタン押下時の処理
 *-------------------------------------------------------------------*/
function getSchema() {
	editResultTable(executeGetSchema(), false);
}
//接続文字列を生成する
function generateConnectString() {
	return 'PROVIDER=MSDASQL'
		+ ';DRIVER={' + currentOdbcItem.value + '}'
		+ ';DBQ=' + targetFullPath.value
		+ ';HDR=' + (checkBoxHeaderColumn.checked ? 'Yes' : 'No')
		+ ';ReadOnly=' + (checkBoxReadOnly.checked ? '1' : '0')
		+ ';';
}
//テーブル名一覧でダブルクリックしたら、SQLエリアに挿入
function pickupTableName() {
	var tableNameObj = dom_getSelectedOption(null, 'tableNamesArea');
	if (!tableNameObj) return;
	var tableName = '[' + tableNameObj.value + ']';
	sqlTextArea.focus(); //sqlTextAreaが選択されてない場合は要フォーカス
	var textRange = sqlTextArea.document.selection.createRange(); //IEのみ有効の方法
	textRange.text = tableName;
}
